//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/InstrumentsSet.cpp
//! @brief     Implement class InstrumentsSet
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Device/InstrumentsSet.h"
#include "GUI/Model/Device/BackgroundItems.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Support/Util/String.h"
#include <QUuid>

namespace {
namespace Tag {

const QString Instrument("Instrument");
const QString CurrentIndex("CurrentIndex");

} // namespace Tag
} // namespace

InstrumentsSet::InstrumentsSet() = default;

InstrumentsSet::~InstrumentsSet() = default;

void InstrumentsSet::clear()
{
    m_instruments.clear();
}

void InstrumentsSet::writeTo(QXmlStreamWriter* w) const
{
    XML::writeAttribute(w, XML::Attrib::version, uint(1));

    // instruments
    for (const auto& sel : m_instruments) {
        w->writeStartElement(Tag::Instrument);
        sel.writeTo(w);
        w->writeEndElement();
    }

    // selected index
    w->writeStartElement(Tag::CurrentIndex);
    XML::writeAttribute(w, XML::Attrib::value, m_current_index);
    w->writeEndElement();
}

void InstrumentsSet::readFrom(QXmlStreamReader* r)
{
    m_instruments.clear();

    const uint version = XML::readUIntAttribute(r, XML::Attrib::version);
    Q_UNUSED(version)

    while (r->readNextStartElement()) {
        QString tag = r->name().toString();

        // instrument
        if (tag == Tag::Instrument) {
            PolyItem<InstrumentsCatalog> p;
            p.readFrom(r);
            m_instruments.emplace_item_back(p.releaseItem());
            XML::gotoEndElementOfTag(r, tag);

            // selected index
        } else if (tag == Tag::CurrentIndex) {
            XML::readAttribute(r, XML::Attrib::value, &m_current_index);
            XML::gotoEndElementOfTag(r, tag);

        } else
            r->skipCurrentElement();
    }
}

void InstrumentsSet::emplace_back(InstrumentItem* item)
{
    m_instruments.emplace_item_back(item);
}

InstrumentItem* InstrumentsSet::insertItemCopy(const InstrumentItem& source)
{
    InstrumentItem* copy = source.createItemCopy();
    copy->setId(QUuid::createUuid().toString());
    emplace_back(copy);
    return copy;
}

QVector<InstrumentItem*> InstrumentsSet::instrumentItems() const
{
    QVector<InstrumentItem*> output;
    for (const auto& sel : m_instruments)
        output.append(sel.certainItem());
    return output;
}

QStringList InstrumentsSet::instrumentNames() const
{
    QStringList existingNames;
    for (const auto* item : instrumentItems())
        existingNames << item->instrumentName();
    return existingNames;
}

QString InstrumentsSet::suggestInstrumentName(const QString& baseName) const
{
    return GUI::Util::String::suggestName(instrumentNames(), baseName);
}

QVector<InstrumentItem*> InstrumentsSet::instrument2DItems() const
{
    QVector<InstrumentItem*> result;
    for (auto* p : instrumentItems())
        if (auto* p2D = dynamic_cast<InstrumentItem*>(p))
            result << p2D;
    return result;
}

InstrumentItem* InstrumentsSet::findInstrumentItemById(const QString& instrumentId) const
{
    for (auto* instrument : instrumentItems())
        if (instrument->id() == instrumentId)
            return instrument;
    return nullptr;
}

bool InstrumentsSet::instrumentExists(const QString& instrumentId) const
{
    return findInstrumentItemById(instrumentId) != nullptr;
}

void InstrumentsSet::removeInstrument(InstrumentItem* instrument)
{
    m_instruments.erase_item(instrument);
    emit instrumentAddedOrRemoved();
}

void InstrumentsSet::setInstrumentName(InstrumentItem* instrument, const QString& name)
{
    if (instrument->instrumentName() != name) {
        instrument->setInstrumentName(name);
        emit instrumentNameChanged(instrument);
        emit instrumentChanged(instrument);
    }
}
